@@ -46,6 +46,9 @@ module Kramdown
         @toc_code = nil
         @indent = 2
         @stack = []
+
+        # stash string representation of symbol to avoid allocations from multiple interpolations.
+        @highlighter_class = " highlighter-#{options[:syntax_highlighter]}"
       end
 
       # The mapping of element type to conversion method.
@@ -78,7 +81,8 @@ module Kramdown
       end
 
       def convert_text(el, _indent)
-        escape_html(el.value, :text)
+        escaped = escape_html(el.value, :text)
+        @options[:remove_line_breaks_for_cjk] ? fix_cjk_line_break(escaped) : escaped
       end
 
       def convert_p(el, indent)
@@ -88,7 +92,7 @@ module Kramdown
             el.children.first.options[:ial]&.[](:refs)&.include?('standalone')
           convert_standalone_image(el.children.first, indent)
         else
-          format_as_block_html(el.type, el.attr, inner(el, indent), indent)
+          format_as_block_html("p", el.attr, inner(el, indent), indent)
         end
       end
 
@@ -135,7 +139,7 @@ module Kramdown
       end
 
       def convert_blockquote(el, indent)
-        format_as_indented_block_html(el.type, el.attr, inner(el, indent), indent)
+        format_as_indented_block_html("blockquote", el.attr, inner(el, indent), indent)
       end
 
       def convert_header(el, indent)
@@ -152,12 +156,15 @@ module Kramdown
         "#{' ' * indent}<hr#{html_attributes(el.attr)} />\n"
       end
 
+      ZERO_TO_ONETWENTYEIGHT = (0..128).to_a.freeze
+      private_constant :ZERO_TO_ONETWENTYEIGHT
+
       def convert_ul(el, indent)
         if !@toc_code && (el.options[:ial][:refs].include?('toc') rescue nil)
-          @toc_code = [el.type, el.attr, (0..128).to_a.map { rand(36).to_s(36) }.join]
+          @toc_code = [el.type, el.attr, ZERO_TO_ONETWENTYEIGHT.map { rand(36).to_s(36) }.join]
           @toc_code.last
         elsif !@footnote_location && el.options[:ial] && (el.options[:ial][:refs] || []).include?('footnotes')
-          @footnote_location = (0..128).to_a.map { rand(36).to_s(36) }.join
+          @footnote_location = ZERO_TO_ONETWENTYEIGHT.map { rand(36).to_s(36) }.join
         else
           format_as_indented_block_html(el.type, el.attr, inner(el, indent), indent)
         end
@@ -165,7 +172,7 @@ module Kramdown
       alias convert_ol convert_ul
 
       def convert_dl(el, indent)
-        format_as_indented_block_html(el.type, el.attr, inner(el, indent), indent)
+        format_as_indented_block_html("dl", el.attr, inner(el, indent), indent)
       end
 
       def convert_li(el, indent)
@@ -188,7 +195,7 @@ module Kramdown
             break
           end
         end if !attr['id'] && @stack.last.options[:ial] && @stack.last.options[:ial][:refs]
-        format_as_block_html(el.type, attr, inner(el, indent), indent)
+        format_as_block_html("dt", attr, inner(el, indent), indent)
       end
 
       def convert_html_element(el, indent)
@@ -263,7 +270,7 @@ module Kramdown
       end
 
       def convert_a(el, indent)
-        format_as_span_html(el.type, el.attr, inner(el, indent))
+        format_as_span_html("a", el.attr, inner(el, indent))
       end
 
       def convert_img(el, _indent)
@@ -296,7 +303,7 @@ module Kramdown
           @footnotes << [name, el.value, number, 0]
           @footnotes_by_name[name] = @footnotes.last
         end
-        "<sup id=\"fnref:#{name}#{repeat}\">" \
+        "<sup id=\"fnref:#{name}#{repeat}\" role=\"doc-noteref\">" \
           "<a href=\"#fn:#{name}\" class=\"footnote\">" \
           "#{number}</a></sup>"
       end
@@ -400,7 +407,7 @@ module Kramdown
       # Add the syntax highlighter name to the 'class' attribute of the given attribute hash. And
       # overwrites or add a "language-LANG" part using the +lang+ parameter if +lang+ is not nil.
       def add_syntax_highlighter_to_class_attr(attr, lang = nil)
-        (attr['class'] = (attr['class'] || '') + " highlighter-#{@options[:syntax_highlighter]}").lstrip!
+        (attr['class'] = (attr['class'] || '') + @highlighter_class).lstrip!
         attr['class'].sub!(/\blanguage-\S+|(^)/) { "language-#{lang}#{$1 ? ' ' : ''}" } if lang
       end
 
@@ -475,9 +482,9 @@ module Kramdown
         result
       end
 
-      FOOTNOTE_BACKLINK_FMT = "%s<a href=\"#fnref:%s\" class=\"reversefootnote\">%s</a>"
+      FOOTNOTE_BACKLINK_FMT = "%s<a href=\"#fnref:%s\" class=\"reversefootnote\" role=\"doc-backlink\">%s</a>"
 
-      # Return a HTML ordered list with the footnote content for the used footnotes.
+      # Return an HTML ordered list with the footnote content for the used footnotes.
       def footnote_content
         ol = Element.new(:ol)
         ol.attr['start'] = @footnote_start if @footnote_start != 1
@@ -485,7 +492,7 @@ module Kramdown
         backlink_text = escape_html(@options[:footnote_backlink], :text)
         while i < @footnotes.length
           name, data, _, repeat = *@footnotes[i]
-          li = Element.new(:li, nil, 'id' => "fn:#{name}")
+          li = Element.new(:li, nil, 'id' => "fn:#{name}", 'role' => 'doc-endnote')
           li.children = Marshal.load(Marshal.dump(data.children))
 
           para = nil
@@ -520,7 +527,7 @@ module Kramdown
         if ol.children.empty?
           ''
         else
-          format_as_indented_block_html('div', {class: "footnotes"}, convert(ol, 2), 0)
+          format_as_indented_block_html('div', {class: "footnotes", role: "doc-endnotes"}, convert(ol, 2), 0)
         end
       end
 
